@@ -1,7 +1,7 @@
 /************************************************************************
  * linux.c                                                              *
  *                                                                      *
- * Copyright (C) 2002 - 2020 Joshua Birnbaum <engineer@noorg.org>.      *
+ * Copyright (C) 2002 - 2023 Joshua Birnbaum <engineer@noorg.org>.      *
  * All Rights Reserved.                                                 *
  *                                                                      *
  * This program is free software; you can redistribute it and/or modify *
@@ -31,7 +31,7 @@
  * Project ifchk -- Host based network interface promiscuous   *
  *                  mode detection and handling.               *
  *                                                             *
- * Version:         1.1.2                                      *
+ * Version:         1.1.3                                      *
  *                                                             *
  * ----------------------------------------------------------- *
  *                                                             *
@@ -79,13 +79,16 @@ int ifStat( void )
     char *bondPtr = NULL;                   /* Pathname of bond interface sysfs object to query. */
     char slaveFile[BONDBUFSZ] = "";         /* Pathname of sysfs filesystem bonding interface slaves. */
     char actSlave[BONDBUFSZ] = "";          /* Pathname of sysfs filesystem active slave interface. */
-    FILE *bondslavefd = NULL;               /* File pointer that points to bonding interface slaves. */
+    char slaves[BONDBUFSZ] = "";            /* Bond interface slaves + currently active slave. */
+    char *slaveData = NULL;                 /* For getSlaves(). */
     char buf[BONDBUFSZ] = "";               /* Buffer for sysfs filesystem bonding interface slaves. */
     char slaveBuf[BONDBUFSZ] = "";          /* Buffer for sysfs filesystem binding active slave interface. */
-    char *nlPtr = NULL;                     /* Ptr for trailing newline processing. */
+    char *charPtr = NULL;                   /* For actSlave. */
+    char *wsPtr = NULL;                     /* Ptr for trailing white space processing. */
     int ifCount = 0;                        /* Interface count. */
     struct sysfs_attribute *sysAttr;        /* sysfs attribute to open (e.g., /sys/class/net/eth0/flags). */
     char *endPtr = NULL;                    /* For strtol(). */
+    char *eolPtr = NULL;                    /* For slave interface + active slave string processing. */
     int i = 0;                              /* For ifMetric(). */
     long j = 0;                             /* Interface flags mask. */
     int sockfd = 0;                         /* Socket descriptor for ioctl(). */
@@ -95,29 +98,9 @@ int ifStat( void )
     struct ifList *new = NULL;              /* Constructed locally & passed \ */
     struct ifList *cur = NULL;              /* to ifMetric() for processing \ */
     struct ifList *freeList = NULL;         /* & output. */
-    struct fsObj slaveObject;               /* File attribute checking policy. */
-
-    /*
-     * Reset all statFile() file checks.
-     */
-    slaveObject.openRead = FALSE;
-    slaveObject.procLen = FALSE;
-    slaveObject.procOwner = FALSE;
-    slaveObject.procMode = FALSE;
-    slaveObject.regLen = FALSE;
-    slaveObject.regMode = FALSE;
 
     memset( mntPath, '\000', sizeof( mntPath ) );
 
-    /*
-     * Preset the file query policy for fileStat(), below. Specifically,
-     * we want to probe the sysfs file attributes of bonding interface
-     * slaves (e.g., em1, em2), if any.
-     */
-    slaveObject.openRead = TRUE;
-    slaveObject.regLen = TRUE;
-    slaveObject.regMode = TRUE;
-
     /*
      * Determine the sysfs mount point.
      */
@@ -206,68 +189,32 @@ int ifStat( void )
             strncat( slaveFile, class -> name, sizeof( slaveFile ) );
             strncat( slaveFile, "/bonding/slaves", sizeof( slaveFile ) );
 
-            /*
-             * Prepare to securely open the slave sysfs file.
-             */
-            if( ( bondslavefd = statFile( slaveFile, slaveObject ) ) == NULL )
+            if( ( slaveData = getSlaves( pathName, class -> name, slaveFile ) ) == NULL )
             {
-                return (-1);
+                errHandler( "ERROR: could not get %s slave configuration\n", class -> name );
+                exit( EXIT_FAILURE );
              }
 
-            /*
-             * Once opened, read the file's contents (e.g., "em1 em2").
-             */
-            while( ( fgets( buf, sizeof( buf ), bondslavefd ) ) != NULL )
-            {
-                if( ( nlPtr = strchr( buf, '\n' ) ) )
-                {
-                    *nlPtr = '\000';
-                 }
-             }
 
-            /*
-             * Once read, close the slave sysfs file.
-             */
-            if( fclose( bondslavefd ) != 0 )
-            {
-                errHandler( "ERROR: fclose(): could not close %s: %s\n", slaveFile, strerror( errno ) );
-                return (-1);
-             }
+            memset( slaves, '\000', sizeof( slaves ) );
+            memset( actSlave, '\000', sizeof( actSlave ) );
 
-            /*
-             * Build a full path to the active slave interface sysfs file.
-             */
-            strncpy( actSlave, "/sys/devices/virtual/net/", sizeof( actSlave ) );
-            strncat( actSlave, class -> name, sizeof( actSlave ) );
-            strncat( actSlave, "/bonding/active_slave", sizeof( actSlave ) );
+            strncpy( slaves, slaveData, sizeof( slaves ) );
 
-            /*
-             * Prepare to securely open the active slave sysfs file.
-             */
-            if( ( bondslavefd = statFile( actSlave, slaveObject ) ) == NULL )
-            {
-                return (-1);
-             }
+            slaves[BONDBUFSZ - 1] = '\000';
 
-            /*
-             * Once opened, read the file's contents (e.g., "em1").
-             */
-            while( ( fgets( slaveBuf, sizeof( slaveBuf ), bondslavefd ) ) != NULL )
-            {
-                if( ( nlPtr = strchr( slaveBuf, '\n' ) ) )
-                {
-                    *nlPtr = '\000';
-                 }
-             }
+            wsPtr = strrchr( slaves, ' ' );
+            wsPtr++;
 
-            /*
-             * Once read, close the slave sysfs file.
-             */
-            if( fclose( bondslavefd ) != 0 )
-            {
-                errHandler( "ERROR: fclose(): could not close %s: %s\n", slaveFile, strerror( errno ) );
-                return (-1);
-             }
+            charPtr = actSlave;
+
+            memcpy( charPtr, wsPtr, sizeof( wsPtr) );
+
+            wsPtr = strrchr( slaves, ' ' );
+
+            eolPtr = strchr( slaves, '\000' );
+
+            memmove( wsPtr, eolPtr, 1 );
          }
 
         /*
@@ -308,9 +255,9 @@ int ifStat( void )
         {
             if( bFlag == TRUE )
             {
-                printf( "%6s: PROMISC (slave: %s)\n", class -> name, buf );
+                printf( "%6s: PROMISC (slave: %s)\n", class -> name, slaves );
 
-                if( ( writeLog( LOG_WARNING, "%4s: PROMISC (slave: %s)\n", class -> name, buf ) ) != 0 )
+                if( ( writeLog( LOG_WARNING, "%4s: PROMISC (slave: %s)\n", class -> name, slaves ) ) != 0 )
                 {
                     return (-1);
                  }
@@ -367,9 +314,9 @@ int ifStat( void )
 
             if( bFlag == TRUE )
             {
-                printf( "%6s: PROMISC [*] (slave: %s)\n", class -> name, buf );
+                printf( "%6s: PROMISC [*] (slave: %s)\n", class -> name, slaves );
 
-                if( ( writeLog( LOG_WARNING, "%4s: PROMISC [*] (slave: %s)\n", class -> name, buf ) ) != 0 )
+                if( ( writeLog( LOG_WARNING, "%4s: PROMISC [*] (slave: %s)\n", class -> name, slaves ) ) != 0 )
                 {
                     return (-1);
                  }
@@ -452,10 +399,10 @@ int ifStat( void )
         {
             if( bFlag == TRUE )
             {
-                printf( "%6s: normal <slave: %s, actv, %s>\n", class -> name, buf, slaveBuf );
+                printf( "%6s: normal <slave: %s, actv: %s>\n", class -> name, slaves, actSlave );
 
                 if( ( writeLog( LOG_INFO, "%4s: normal <slave: %s, actv: %s>\n",
-                                               class -> name, buf, slaveBuf ) ) != 0 )
+                                               class -> name, slaves, actSlave ) ) != 0 )
                 {
                     return (-1);
                  }
@@ -480,7 +427,7 @@ int ifStat( void )
             if( bFlag == TRUE )
             {
                 if( ( writeLog( LOG_INFO, "%4s: *down* <slave: %s, actv: %s>\n",
-                                               class -> name, buf, slaveBuf ) ) != 0 )
+                                               class -> name, slaves, actSlave ) ) != 0 )
                 {
                     return (-1);
                  }
